#!/usr/bin/env python
'irc.py - Python IRC Bot, by Sean B. Palmer'

import sys, socket, asyncore, asynchat

def ensure_data(data, disallowed, strict): 
   for byte in disallowed: 
      if strict and (byte in data): 
         raise ValueError('Disallowed byte: ' + str(byte))
      data = data.replace(byte, b'')
   return data

def decode(data): 
   for encoding in ('utf-8', 'iso-8859-1', 'cp1252'): 
      try: return data.decode(encoding)
      except UnicodeDecodeError: continue

def bytes(string): 
   return set(list(string))

class Client(asynchat.async_chat): 
   def __init__(self, nick, strict=False): 
      asynchat.async_chat.__init__(self)
      self.set_terminator('\n')
      self.collect_incoming_data = self._collect_incoming_data
      self.nick = nick
      self.strict = strict

   def run(self, host, port=6667): 
      self.create_socket(socket.AF_INET, socket.SOCK_STREAM)
      self.connect((host, port))
      asyncore.loop()

   def found_terminator(self): 
      data = self._get_data()
      data = data.rstrip('\r')

      if data.startswith(':'): 
         sender, data = data.split(' ', 1)
      else: sender = ':'
      parts = [decode(p) for p in data.split(' :', 1)]
      parts = parts[0].split(' ') + parts[1:]
      self.dispatch(parts[0], decode(sender[1:]), *parts[1:])

   def dispatch(self, command, sender, *args): 
        ## DO THINGS HERE

        #('RECEIVED:', u'PRIVMSG', u'pgadey!~pgadey@127.0.0.1', (u'empat', u'\x01PING 1439699777 744712\x01'))

        print('RECEIVED:', command, sender, args)

        if 'PING' in args[1]:
            ping_resp = args[1][6:]; # six is the length of 'PONG '
            print "Responding with: %s" % ping_resp;
            self.write("PONG %s \r\n" % ping_resp)
            

   def serialise(self, arg, trailing): 
      arg = arg.encode('utf-8')
      arg = ensure_data(arg, bytes('\x00\r\n'), self.strict)
      
      if not trailing: 
         arg = ensure_data(arg, bytes(' :'), self.strict)
      elif (' ' in arg) or (':' in arg): 
         arg = ':' + arg
      return arg

   def write(self, *args): 
      data = [self.serialise(arg, False) for arg in args[:-1]]
      data += [self.serialise(arg, True) for arg in args[-1:]]
      self.send(' '.join(data) + '\r\n')

   def handle_connect(self): 
      self.write('NICK', self.nick)
      self.write('USER', self.nick, '+iw', self.nick, 'irc.py')

   def msg(self, recipient, message): 
      self.write('PRIVMSG', recipient, message)

   def join(self, *channels): 
      for channel in channels: 
        print("Joining %s \n" % channel);
        self.write('JOIN', channel)

if __name__ == '__main__': 
    bot = Client('empat')
    bot.run('irc.ctrl-c.club')
    bot.msg('pgadey', 'howdy')
